@@ -25,22 +25,39 @@ sub check_current($self, $key=undef) {
 
 sub current { Evo::Util::_current('CURRENT', shift->singleton, @_) }
 
+sub _call {
+  my ($self, $cb) = (shift, pop);
+  my $single = $self->singleton;
+
+  do { $self->{dsl_depth}++; $single->{dsl_global_depth}++ };
+
+  Evo::Util::_guard { $self->{dsl_depth}--; $single->{dsl_global_depth}--; }
+  sub { $cb->() };
+
+
+}
+sub dsl_depth { $_[0]->{dsl_depth} //= 0 }
+sub dsl_global_depth { $_[0]->singleton->{dsl_global_depth} //= 0 }
+
 # extend current for next cb. shoult die on existing keys
 sub dsl_extend {
   my ($self, $cb) = (shift, pop);
-  my $extend  = shift;
+  my $extend  = shift // Carp::croak 'provide extend hashref';
+  my @args    = @_;
   my $current = {$self->current->%*};
 
   local $self->singleton->{CURRENT} = $current;
   $self->current($_ => $extend->{$_}) for keys %$extend;
-  $cb->(@_);
+  $self->_call(sub { $cb->(@args) });
 }
 
 # initialize dsl
 sub dsl_call {
   my ($self, $cb) = (shift, pop);
   local $self->singleton->{CURRENT} = shift // {};
-  $cb->(@_);
+  my @args = @_;
+
+  $self->_call(sub { $cb->(@args) });
 }
 
 sub run_wrappers { Evo::Util::run_wrappers(@_) }
@@ -70,7 +87,7 @@ Evo::Manager - Perl Evo manager
 
 =head1 VERSION
 
-version 0.008
+version 0.009
 
 =head1 SYNOPSIS
 
@@ -180,6 +197,15 @@ Singleton - the same instance will be returned for every invocation
 
   my $single = Evo::Manager->singleton;  
 
+=head2 dsl_depth
+
+  A depth of recurcive L</"dsl_call"> or L</"dsl_extend"> invocations by
+  instance. C<0> means we are not in the dsl
+
+=head2 dsl_global_depth
+
+  The same as L</"dsl_depth"> but calculates all invocations of all instances.
+
 =head1 ATTENTION
 
 Curry isn't a real currying function, it's a partial function. But I have no
