@@ -3,13 +3,6 @@ use Evo::Base -base;
 
 sub cleanup($self) { delete $self->{$_} for $self->cleanup_list }
 
-sub try_or_cleanup($self, $code) {
-  local $@;
-  my @res = eval { $code->() };
-  do { $self->cleanup; die $@ } if $@;
-  @res;
-}
-
 state $CLEANUP_ATTRS = {};
 
 sub register_cleanup($class, $attr) {
@@ -63,7 +56,7 @@ Railway::Cleanup - autocleaner
 
 =head1 VERSION
 
-version 0.008
+version 0.009
 
 =head1 METHODS
 
@@ -71,12 +64,6 @@ version 0.008
 
 cleanup attributes
 
-=head2 try_or_cleanup
-
-  $o->try_or_cleanup(sub { die });
-
-Execute a code and invoke cleanup if code raised an exception
-
 =head2 register_cleanup
 
 register a keys for cleanup
