@@ -1,5 +1,6 @@
 package Railway::Train;
 use Evo::Base 'Railway::Cleanup';
+use Evo::Util;
 
 has 'manager';
 
@@ -29,7 +30,9 @@ sub run($self) {
     $self->depart;
   }
 
-  do { $self->finish; $self->cleanup; $self; };
+  Evo::Util::_guard { $self->cleanup }
+  sub { $self->done && $self->done->($self) };
+  return;
 }
 
 # prepare train or die
@@ -52,14 +55,9 @@ sub can_run($self) {
   $self->{index} < $way->stations->@*;
 }
 
-sub finish ($self) {
-  return unless my $done = $self->done;
-  $self->try_or_cleanup(sub { $done->($self) });
-}
-
 # invoke or cleanup
 sub handle_station($self, $station) {
-  $self->try_or_cleanup(sub { $station->($self) });
+  Evo::Util::_guard_on_die { $self->cleanup } sub { $station->($self) };
 }
 
 # leave current station
@@ -87,7 +85,7 @@ Railway::Train - a running engine
 
 =head1 VERSION
 
-version 0.008
+version 0.009
 
 =head1 SYNOPSIS
 
